# -*- coding: utf-8 -*-
#!/usr/env python3

# Copyright (C) 2018-2019 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2018-2019"
__credits__ = ""
__license__ = "GPL3"
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"

name = "pyDSA_gui"



# -*- coding: utf-8 -*-
#!/usr/env python3

# Copyright (C) 2018-2019 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2018-2019"
__credits__ = ""
__license__ = "GPL3"
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as Canvas
from matplotlib.backends.backend_qt5 import (
    NavigationToolbar2QT as MplNavigationBar)
from PyQt5.QtWidgets import QWidget, QVBoxLayout
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
import numpy as np
from pyDSA_core.baseline import Baseline

from .mpl_handlers import BaselineHandler, RectangleHandler, ScalingHandler, \
    VerticalLineHandler

colors = {'baseline': 'tab:blue',
          'crop_area': 'tab:red',
          'scaling': 'tab:green',
          'edge': 'tab:green',
          'fit': 'tab:orange',
          'ca': 'tab:blue',
          'plot1': 'tab:blue',
          'plot2': 'tab:orange',
          'figure_background': '#FAFAFA',
          'vertical line': 'tab:green'}


# class MplPlotWidget(Canvas):
#     def __init__(self, parent=None):
#         super(MplPlotWidget, self).__init__(Figure())

#         self.setParent(parent)
#         self.figure = Figure(dpi=100)
#         self.canvas = Canvas(self.figure)
#         self.ax = self.figure.add_subplot(111)
#         self.plot = self.ax.plot([])[0]
#         self.xmin = 0
#         self.xmax = 1

#     def plotSomeDataPoints(self, nmbpoints):
#         x = np.linspace(0, 10, nmbpoints)
#         y = np.cos(x)
#         self.plot.set_data([x, y])
#         self.draw()

#     def setProperty(self, prop, val):
#         if prop in ['xmin', 'xmax']:
#             setattr(self, prop, val)
#             self.ax.set_xlim(self.xmin, self.xmax)
#             self.draw()
#         else:
#             super(MplPlotWidget, self).setProperty(prop, val)

#     def set_xmax(self, xmax):
#         self.xmax = xmax
#         self.ax.set_xlim(self.xmin, self.xmax)
#         self.draw()


class MplToolBar(MplNavigationBar):
    toolitems = (
        ('Home', 'Reset original view', 'home', 'home'),
        ('Back', 'Back to previous view', 'back', 'back'),
        ('Forward', 'Forward to next view', 'forward', 'forward'),
        (None, None, None, None),
        ('Pan', 'Pan axes with left mouse, zoom with right', 'move', 'pan'),
        ('Zoom', 'Zoom to rectangle', 'zoom_to_rect', 'zoom'),
        # ('Subplots', 'Configure subplots', 'subplots', 'configure_subplots'),
        # (None, None, None, None),
        # ('Save', 'Save the figure', 'filesave', 'save_figure'),
    )


class MplCanvas(Canvas):
    def __init__(self, parent=None, fill=False):
        # Plot
        self.figure = Figure(dpi=100, figsize=(200, 200),
                             facecolor=colors['figure_background'])
        super(MplCanvas, self).__init__(self.figure)
        self.setParent(parent)
        self.parent = parent
        if fill:
            self.ax = self.figure.add_axes([0, 0, 1, 1])
        else:
            self.ax = self.figure.subplots(1, 1)


class MplWidget(QWidget):
    def __init__(self, *args, **kwargs):
        fill = kwargs.pop('fill')
        QWidget.__init__(self, *args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.canvas = MplCanvas(parent=self, fill=fill)
        self.toolbar = MplToolBar(self.canvas, self)
        self.layout().addWidget(self.toolbar)
        self.layout().addWidget(self.canvas)
        self.figure = self.canvas.figure
        self.ax = self.canvas.ax

    def reset_zoom(self):
        self.toolbar.home()


class MplWidgetImport(MplWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(fill=True, *args, **kwargs)
        # Image
        self.data = np.random.rand(200, 300)
        self.im = self.ax.imshow(self.data,
                                 interpolation='None',
                                 cmap=plt.cm.binary_r)
        # add handlers
        # Cropped area
        self.rect_hand = RectangleHandler(self.canvas, self.figure, self.ax)
        # Baseline
        self.baseline_hand = BaselineHandler(self.canvas, self.figure, self.ax)
        # Scaling
        self.is_scaling = False
        self.scaling_hand = ScalingHandler(self.canvas, self.figure, self.ax)
        # Clean stuff !
        self.ax.set_xticks([])
        self.ax.set_xticklabels([])
        self.ax.set_yticks([])
        self.ax.set_yticklabels([])
        # Connect event handlerd
        self.connect_press = self.canvas.mpl_connect('button_press_event',
                                                     self.on_press)
        self.connect_release = self.canvas.mpl_connect('button_release_event',
                                                       self.on_release)
        self.connect_motion = self.canvas.mpl_connect('motion_notify_event',
                                                      self.on_motion)

    def update_image(self, im, replot=False):
        self.data = im.transpose()[::-1]
        if replot:
            self.im = self.ax.imshow(self.data,
                                     interpolation="None",
                                     cmap=plt.cm.binary_r)
            # clean stuff !
            self.ax.set_xticks([])
            self.ax.set_xticklabels([])
            self.ax.set_yticks([])
            self.ax.set_yticklabels([])
        else:
            self.im.set_data(self.data)
        self.canvas.draw()

    def update_crop_area(self, xlim1, xlim2, ylim1, ylim2):
        self.rect_hand.update_lims(xlim1, xlim2,
                                   ylim1, ylim2)
        self.canvas.draw()

    def update_baseline(self, pt1, pt2):
        self.baseline_hand.update_pts(pt1, pt2)
        self.canvas.draw()

    def update_scaling_pts(self, pts):
        self.scaling_hand.update_pts(pts)
        self.canvas.draw()

    def get_scale(self):
        return self.scaling_hand.get_scale()

    def on_press(self, event):
        # see: https://stackoverflow.com/questions/28001655/draggable-line-with-draggable-points
        # Check
        if event.inaxes != self.ax:
            return None
        # toolbar want the focus !
        if self.toolbar._active is not None:
            return None
        # On a scaling handle
        if self.scaling_hand.select_hand_at_point(event):
            self.scaling_hand.prepare_for_drag()
        # Add a new scaling point
        elif self.is_scaling and len(self.scaling_hand.pts) < 2:
            self.scaling_hand.add_point(event)
            self.canvas.draw()
        # On a crop handle
        elif self.rect_hand.select_hand_at_point(event):
            self.rect_hand.prepare_for_drag()
        # On a baseline handle
        elif self.baseline_hand.select_hand_at_point(event):
            self.baseline_hand.prepare_for_drag()

    def on_motion(self, event):
        # Check
        if event.inaxes != self.ax:
            return None
        #
        if self.scaling_hand.dragged_hand is not None:
            self.scaling_hand.drag_to(event)
        elif self.rect_hand.dragged_hand is not None:
            self.rect_hand.drag_to(event)
        elif self.baseline_hand.dragged_hand is not None:
            self.baseline_hand.drag_to(event)

    def on_release(self, event):
        if self.scaling_hand.is_dragging():
            self.scaling_hand.finish_drag()
            self.scaling_hand.unselect_hand()
        elif self.rect_hand.is_dragging():
            self.rect_hand.finish_drag()
            self.rect_hand.unselect_hand()
        elif self.baseline_hand.is_dragging():
            self.baseline_hand.finish_drag()
            self.baseline_hand.unselect_hand()


class MplWidgetDetect(MplWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(fill=True, *args, **kwargs)
        #
        self.axbackground = None
        self.need_replot = True
        self.tab_opened = False
        # Image
        self.data = np.random.rand(200, 300)
        self.im = self.ax.imshow(self.data,
                                 interpolation="None",
                                 cmap=plt.cm.binary_r)
        # baseline
        self.baseline = self.ax.plot([0, 0], [0, 0],
                                     color=colors['baseline'],
                                     lw=0.5,
                                     ls="-")[0]
        # edges
        self.edge = self.ax.plot([0], [0],
                                 color=colors['edge'],
                                 marker='o',
                                 ms=3,
                                 alpha=0.5,
                                 ls='none')[0]
        # Clean stuff !
        self.ax.set_xticks([])
        self.ax.set_xticklabels([])
        self.ax.set_yticks([])
        self.ax.set_yticklabels([])

    def update_image(self, im, blit=True):
        new_data = im.transpose()[::-1]
        if not np.all(new_data.shape == self.data.shape):
            self.need_replot = True
        self.data = new_data
        self.im.set_data(self.data)
        if blit:
            self.blit_it()

    def update_baseline(self, pt1, pt2, blit=True):
        sizex = abs(self.ax.viewLim.width)
        pt1, pt2 = Baseline.get_baseline_from_points([pt1, pt2],
                                                     xmin=-100*sizex,
                                                     xmax=100*sizex)
        self.baseline.set_data([[pt1[0], pt2[0]],
                                [pt1[1], pt2[1]]])
        if blit:
            self.blit_it()

    def update_edge(self, edge, blit=True):
        self.edge.set_data(*edge)
        if blit:
            self.blit_it()

    def blit_it(self):
        if not self.tab_opened:
            # If tab is not open, just redispay the data
            self.im = self.ax.imshow(self.data,
                                     interpolation='None',
                                     cmap=plt.cm.binary_r)
            return None
        if self.need_replot or self.axbackground is None:
            self.im = self.ax.imshow(self.data,
                                     interpolation='None',
                                     cmap=plt.cm.binary_r)
            # clean stuff !
            self.ax.set_xticks([])
            self.ax.set_xticklabels([])
            self.ax.set_yticks([])
            self.ax.set_yticklabels([])
            self.canvas.draw()
            self.axbackground = self.canvas.copy_from_bbox(self.ax.bbox)
            self.ax.draw_artist(self.edge)
            self.ax.draw_artist(self.baseline)
            self.need_replot = False
        else:
            self.canvas.restore_region(self.axbackground)
            self.ax.draw_artist(self.im)
            self.ax.draw_artist(self.edge)
            self.ax.draw_artist(self.baseline)
            self.canvas.blit(self.ax.bbox)


class MplWidgetFit(MplWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(fill=True, *args, **kwargs)
        #
        self.axbackground = None
        self.need_replot = True
        self.tab_opened = False
        # Image
        self.data = np.random.rand(200, 300)
        self.im = self.ax.imshow(self.data,
                                 interpolation="None",
                                 cmap=plt.cm.binary_r)
        # baseline
        self.baseline = self.ax.plot([0, 0], [0, 0],
                                     color=colors['baseline'],
                                     lw=0.5,
                                     ls="-")[0]
        # edges
        self.fit = self.ax.plot([0], [0],
                                color=colors['fit'],
                                ls='-')[0]
        self.fit_center = self.ax.plot([0], [0],
                                       color=colors['fit'],
                                       marker='o',
                                       ls='none')[0]
        # contact angles
        self.ca_r = self.ax.plot([], [],
                                 color=colors['ca'],
                                 marker='o',
                                 ms=3,
                                 ls='-')[0]
        self.ca_l = self.ax.plot([], [],
                                 color=colors['ca'],
                                 marker='o',
                                 ms=3,
                                 ls='-')[0]
        # Clean stuff !
        self.ax.set_xticks([])
        self.ax.set_xticklabels([])
        self.ax.set_yticks([])
        self.ax.set_yticklabels([])

    def update_image(self, im, blit=True):
        new_data = im.transpose()[::-1]
        if not np.all(new_data.shape == self.data.shape):
            self.need_replot = True
        self.data = new_data
        self.im.set_data(self.data)
        if blit:
            self.blit_it()

    def update_baseline(self, pt1, pt2, blit=True):
        sizex = abs(self.ax.viewLim.width)
        pt1, pt2 = Baseline.get_baseline_from_points([pt1, pt2],
                                                     xmin=-100*sizex,
                                                     xmax=100*sizex)
        self.baseline.set_data([[pt1[0], pt2[0]],
                                [pt1[1], pt2[1]]])
        if blit:
            self.blit_it()

    def update_fit_and_cas(self, fit, fit_center, cas, blit=True):
        # fit
        if fit_center is None:
            fit_center = [[0], [0]]
        self.fit.set_data(*fit)
        self.fit_center.set_data(fit_center)
        # cas
        ca1, ca2 = cas
        self.ca_r.set_data(*ca1)
        self.ca_l.set_data(*ca2)
        if blit:
            self.blit_it()

    def blit_it(self):
        if not self.tab_opened:
            # If tab is not open, just redispay the data
            self.im = self.ax.imshow(self.data,
                                     interpolation='None',
                                     cmap=plt.cm.binary_r)
            return None
        if self.need_replot or self.axbackground is None:
            self.im = self.ax.imshow(self.data,
                                     interpolation="None",
                                     cmap=plt.cm.binary_r)
            # clean stuff !
            self.ax.set_xticks([])
            self.ax.set_xticklabels([])
            self.ax.set_yticks([])
            self.ax.set_yticklabels([])
            self.canvas.draw()
            self.axbackground = self.canvas.copy_from_bbox(self.ax.bbox)
            self.ax.draw_artist(self.baseline)
            self.ax.draw_artist(self.fit)
            self.ax.draw_artist(self.fit_center)
            self.ax.draw_artist(self.ca_r)
            self.ax.draw_artist(self.ca_l)
            self.need_replot = False
        else:
            self.canvas.restore_region(self.axbackground)
            self.ax.draw_artist(self.im)
            self.ax.draw_artist(self.baseline)
            self.ax.draw_artist(self.fit)
            self.ax.draw_artist(self.fit_center)
            self.ax.draw_artist(self.ca_r)
            self.ax.draw_artist(self.ca_l)
            self.canvas.blit(self.ax.bbox)


class MplWidgetAnalyze(MplWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(fill=False, *args, **kwargs)
        # Figure and axis
        self.ax.yaxis.label.set_color(colors['plot1'])
        self.ax.tick_params(axis='y', colors=colors['plot1'])
        self.ax2 = self.ax.twinx()
        self.ax2.yaxis.label.set_color(colors['plot2'])
        self.ax2.tick_params(axis='y', colors=colors['plot2'])
        self.ax3 = self.ax.twinx()
        self.ax3.set_zorder(1)
        self.ax3.set_xticks([])
        self.ax3.set_yticks([])
        # plots
        self.current_x = []
        self.current_xname = None
        self.current_y = []
        self.current_y2 = []
        self.plot1 = None
        self.plot2 = None
        self.plot1_orig = None  # In case we smooth the other one
        self.plot2_orig = None
        self.indicator1 = None
        self.indicator2 = None
        # Vertical line
        self.vertical_line = VerticalLineHandler(
            self.canvas,
            self.figure,
            self.ax3,
            color=colors['vertical line'])
        # Connect event handler
        self.connect_press = self.canvas.mpl_connect('button_press_event',
                                                     self.on_press)
        self.connect_release = self.canvas.mpl_connect('button_release_event',
                                                       self.on_release)
        self.connect_motion = self.canvas.mpl_connect('motion_notify_event',
                                                      self.on_motion)
        # grid
        self.ax.grid()

    def on_press(self, event):
        # Check
        if event.inaxes != self.ax3:
            return None
        # toolbar want the focus !
        if self.toolbar._active is not None:
            return None
        #
        if self.vertical_line.select_hand_at_point(event):
            self.vertical_line.prepare_for_drag()

    def on_motion(self, event):
        # Check
        if event.inaxes != self.ax3:
            return None
        #
        if self.vertical_line.dragged_hand is not None:
            self.vertical_line.drag_to(event)

    def on_release(self, event):
        if self.vertical_line.is_dragging():
            self.vertical_line.finish_drag()
            self.vertical_line.unselect_hand()

    def update_plots(self, x, y, y2, y_orig, y2_orig,
                     xname, yname, y2name, same_y_lims=False,
                     draw=True, replot=False):
        # check
        if np.all(np.isnan(x)) and np.all(np.isnan(y)):
            x = []
            y = []
            y2 = []
            y_orig = []
            y2_orig = []
        # Because of issues with the vertical line hand...
        if draw:
            self.canvas.draw()
        # store
        self.current_x = x
        self.current_xname = xname
        self.current_y = y
        self.current_y2 = y2
        # check
        if self.plot1 is None:
            replot = True
        # Completely replot is asked
        if replot:
            # Clean
            self.ax.clear()
            self.ax2.clear()
            # Plots
            self.plot1 = self.ax.plot(x, y,
                                      color=colors['plot1'])[0]
            self.plot2 = self.ax2.plot(x, y2,
                                       color=colors['plot2'])[0]
            self.plot1_orig = self.ax.plot(x, y_orig,
                                           color=colors['plot1'],
                                           alpha=0.2)[0]
            self.plot2_orig = self.ax2.plot(x, y2_orig,
                                            color=colors['plot2'],
                                            alpha=0.2)[0]
            self.indicator1 = self.ax.plot([], [], color=colors['plot1'],
                                           marker="o", ls='none')[0]
            self.indicator2 = self.ax2.plot([], [], color=colors['plot2'],
                                            marker="o", ls='none')[0]
            self.ax.grid()
        else:
            # Update plots
            self.plot1.set_data(x, y)
            self.plot2.set_data(x, y2)
            self.plot1_orig.set_data(x, y_orig)
            self.plot2_orig.set_data(x, y2_orig)
        # Hide second axis if necessary
        is_y2 = np.logical_not(np.all(np.isnan(y2)))
        if is_y2:
            self.ax2.set_visible(True)
        else:
            self.ax2.set_visible(False)
        # Update limits
        self.update_lims(x, y, y2, y_orig, y2_orig, samelims=same_y_lims)
        # Update labels
        self.ax.set_xlabel(xname)
        self.ax.set_ylabel(yname)
        if is_y2:
            self.ax2.set_ylabel(y2name)
        else:
            self.ax2.set_ylabel("")
        # Draw if asked
        if draw:
            self.canvas.draw()

    def update_plot_vertical_line(self, ind, draw=True):
        if self.plot1 is None:
            return None
        try:
            x = self.plot1.get_data()[0]
            # Find new position
            if ind is None:
                current_x = None
            else:
                current_x = x[ind]
                if np.isnan(current_x):
                    current_x = None
            # Put the line in the middle if no alid positions
            if current_x is None and len(x) > 0:
                current_x = (np.nanmax(x) + np.nanmin(x))/2
            # Update the line position
            self.vertical_line.update_line_pos(current_x)
        except:
            self.app.log.log_unknown_exception()
        # Draw if asked
        if draw:
            self.canvas.draw()

    def update_lims(self, x, y, y2, y_orig, y2_orig, samelims=False):
        # margin
        margin = 1/50
        # x
        if len(x) <= 1 or np.all(np.isnan(x)):
            x_min, x_max = -1, 1
        else:
            x_min, x_max = np.nanmin(x), np.nanmax(x)
        if np.any(np.isnan([x_min, x_max])):
            x_min = -1
            x_max = 1
        dx = x_max - x_min
        if dx == 0:
            dx = 0.1
        self.ax.set_xlim(x_min - dx*margin, x_max + dx*margin)
        # y
        y = np.concatenate((y, y_orig))
        if len(y) <= 1 or np.all(np.isnan(y)):
            y_min, y_max = -1, 1
        else:
            y_min, y_max = np.nanmin(y), np.nanmax(y)
        if np.any(np.isnan([y_min, y_max])):
            y_min = -1
            y_max = 1
        dy = y_max - y_min
        if dy == 0:
            dy = 0.1
        # y2
        y2 = np.concatenate((y2, y2_orig))
        if len(y2) <= 1 or np.all(np.isnan(y2)):
            y2_min, y2_max = -1, 1
        else:
            y2_min, y2_max = np.nanmin(y2), np.nanmax(y2)
        if np.any(np.isnan([y2_min, y2_max])):
            y2_min = -1
            y2_max = 1
        dy2 = y2_max - y2_min
        if dy2 == 0:
            dy2 = 0.1
        # Set y lims
        if samelims:
            y_min = np.min([y_min, y2_min])
            y_max = np.max([y_max, y2_max])
            dy = np.max([dy, dy2])
            self.ax.set_ylim(y_min - dy*margin, y_max + dy*margin)
            self.ax2.set_ylim(y_min - dy*margin, y_max + dy*margin)
        else:
            self.ax.set_ylim(y_min - dy*margin, y_max + dy*margin)
            self.ax2.set_ylim(y2_min - dy2*margin, y2_max + dy2*margin)
        # dummy ax3
        self.ax3.set_xlim(self.ax.get_xlim())
        self.ax3.set_ylim(self.ax.get_xlim())

    def update_upstream(self):
        """ Update the interface to reflect line modification"""
        # Get the position of the selected points
        xi = self.vertical_line.pt[0]
        x = self.current_x
        y = self.current_y
        y2 = self.current_y2
        yi = None
        y2i = None
        if len(self.current_x) != 0:
            indxs = np.argwhere(np.logical_or(
                np.logical_and(self.current_x[1::] <= xi,
                               self.current_x[:-1] >= xi),
                np.logical_and(self.current_x[1::] >= xi,
                               self.current_x[:-1] <= xi))).flatten()
            yis = [self.linear_interp(x[indx], x[indx + 1],
                                      y[indx], y[indx + 1],
                                      xi)
                   for indx in indxs]
            y2is = [self.linear_interp(x[indx], x[indx + 1],
                                       y2[indx], y2[indx + 1],
                                       xi)
                    for indx in indxs]
            yis = np.asarray(yis)
            y2is = np.asarray(y2is)
            # only keep the one closest to the handler
            vl_pos = self.vertical_line.pt
            vl_pos = self.vertical_line.ax.transData.transform(vl_pos)
            if len(yis) > 0 and not np.all(np.isnan(yis)):
                yvl = self.ax.transData.inverted().transform(vl_pos)[1]
                tmpind = np.nanargmin(abs(yis - yvl))
                yi = yis[tmpind]
            if len(y2is) > 0 and not np.all(np.isnan(y2is)):
                yvl = self.ax2.transData.inverted().transform(vl_pos)[1]
                tmpind = np.nanargmin(abs(y2is - yvl))
                y2i = y2is[tmpind]
        #
        # Update the indicators
        if self.indicator1 is not None:
            if xi is not None and yi is not None:
                self.indicator1.set_data([xi], [yi])
            else:
                self.indicator1.set_data([], [])
        if self.indicator2 is not None:
            if xi is not None and y2i is not None:
                self.indicator2.set_data([xi], [y2i])
            else:
                self.indicator2.set_data([], [])
        # Update interface
        xi_t = ""
        if xi is not None:
            if not np.isnan(xi):
                xi_t = f"{xi:.4f}"
        self.ui.tab4_local_x_value.setText(xi_t)
        yi_t = ""
        if yi is not None:
            if not np.isnan(yi):
                yi_t = f"{yi:.4f}"
        self.ui.tab4_local_y_value.setText(yi_t)
        y2i_t = ""
        if y2i is not None:
            if not np.isnan(y2i):
                y2i_t = f"{y2i:.4f}"
        self.ui.tab4_local_y2_value.setText(y2i_t)
        # Update the current indice
        current_x = self.vertical_line.pt[0]
        if not np.all(np.isnan(x)):
            fn, _, _ = self.app.dsa.get_plotable_quantity("Frame number")
            tmpind = np.nanargmin(abs(current_x - x))
            ind = fn[tmpind] - 1
            self.app.current_ind = ind

    @staticmethod
    def linear_interp(x1, x2, y1, y2, x):
        if x < x1 and x < x2:
            return None
        if x > x2 and x > x1:
            return None
        y = (abs(x1 - x)*y2 + abs(x2 - x)*y1)/abs(x1 - x2)
        return y



# -*- coding: utf-8 -*-
#!/usr/env python3

# Copyright (C) 2018-2019 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2018-2019"
__credits__ = ""
__license__ = "GPL3"
__version__ = ""
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


from datetime import timedelta
from PyQt5 import QtGui
import sys
import traceback
import time


# class catchAndLog(object):
#     """
#     Decorator that catches and log exceptions.
#     """

#     def __init__(self, log, filter=[], default_return=None):
#         """
#         Decorator that catches and log exceptions.

#         Parameters
#         ----------
#         log: Log instance
#             Log instance to communicate with
#         filter: list of Exception
#             Exception to ignore (log on a low level)
#         default_return: anything
#             Default return value for the function in case of Exception
#         """
#         self.log = log
#         self.filter = filter
#         self.default_return = default_return

#     def __call__(self, fun):
#         def wrapped(loc_self, *args):
#             print(f"args: {args}")
#             try:
#                 res = fun(loc_self, *args)
#                 return res
#             except:
#                 exc_info = sys.exc_info()
#                 exc = exc_info[0]
#                 if exc in self.filter:
#                     self.log_low(fun, exc_info)
#                 else:
#                     self.log_high(fun, exc_info)
#                 return self.default_return
#         return wrapped

#     def log_low(self, f, exc_info):
#         pass

#     def log_high(self, f, exc_info):
#         errmess = "".join(traceback.format_exception(*exc_info))
#         mess = f"Unknown error while running {f.__name__}:\n{errmess}"
#         self.log.log(mess, level=3)


class Log(object):
    def __init__(self, ui, delay=1):
        self.logs = []
        self.delay = delay
        self.ui = ui
        self.t0 = time.time()
        self.levels = ['Info', 'Warning', 'Error']
        self.level_colors = [QtGui.QColor(col)
                             for col in ['grey', 'orange', 'red']]

    def log(self, message, level):
        """
        Log a new message.

        Parameters
        ----------
        messages: string
           Message to log.

        level: int in [1, 2, 3]
           Level of the messages:
           1: just log to log tab
           2: log to log tab and display in minibar
           3: log in log tab and display in minibar in red !
        """
        t = str(timedelta(seconds=int(time.time()-self.t0)))
        text = f"{t}  {self.levels[level-1]}: {message}"
        self.logs.append(text)
        # Print in shell
        print(text)
        # Update log tab
        if self.ui.logarea is not None:
            self.ui.logarea.setTextColor(self.level_colors[level - 1])
            self.ui.logarea.append(self.logs[-1])
        # Display in statu bar
        if self.ui.status_bar is not None:
            short_mess = message.split('\n')[-1]
            if level == 2:
                self.ui.status_bar.showMessage(f"{short_mess}", self.delay)
            elif level == 3:
                self.ui.status_bar.showMessage(f"Error: {short_mess}",
                                               self.delay)

    def log_unknown_exception(self):
        # Get last exception
        exc_info = sys.exc_info()
        # log the error
        errmess = "Unknown error: " + str(exc_info[1])
        self.log(errmess, level=3)
        # print traceback
        trcmess = ' '*9 + "".join(traceback.format_exception(*exc_info))
        print(trcmess.replace('\n', '\n' + ' '*9))


if __name__ == "__main__":

    class LowError(Exception):
        pass

    class HighError(Exception):
        pass

    @catchAndLog(Log(None, None, 1), filter=[LowError], default_return=45)
    def foo(a, b):
        if a < 2:
            raise LowError('a is too small')
        if b < 2:
            raise HighError('b is too small')
        return a + b

    # try it
    print("")
    print("Alright")
    print(foo(3, 4))
    print("")
    print("Alright but return 45")
    print(foo(1, 5))
    print("")
    print("Not alright")
    print(foo(3, 1))
    print("")
    print("Not alright")
    print(foo("un", "deux"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'design.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(972, 854)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setObjectName("tabWidget")
        self.tabimport = QtWidgets.QWidget()
        self.tabimport.setObjectName("tabimport")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tabimport)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.frame = QtWidgets.QFrame(self.tabimport)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setMaximumSize(QtCore.QSize(150, 16777215))
        self.frame.setBaseSize(QtCore.QSize(150, 0))
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_2 = QtWidgets.QGroupBox(self.frame)
        self.groupBox_2.setFlat(False)
        self.groupBox_2.setCheckable(False)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.tab1_open_button = QtWidgets.QPushButton(self.groupBox_2)
        self.tab1_open_button.setObjectName("tab1_open_button")
        self.verticalLayout_3.addWidget(self.tab1_open_button)
        self.verticalLayout_2.addWidget(self.groupBox_2)
        self.tab1_crop_box = QtWidgets.QGroupBox(self.frame)
        self.tab1_crop_box.setEnabled(False)
        self.tab1_crop_box.setObjectName("tab1_crop_box")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.tab1_crop_box)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.tab1_zoom_to_area = QtWidgets.QPushButton(self.tab1_crop_box)
        self.tab1_zoom_to_area.setObjectName("tab1_zoom_to_area")
        self.verticalLayout_12.addWidget(self.tab1_zoom_to_area)
        self.tab1_reset_crop = QtWidgets.QPushButton(self.tab1_crop_box)
        self.tab1_reset_crop.setObjectName("tab1_reset_crop")
        self.verticalLayout_12.addWidget(self.tab1_reset_crop)
        self.verticalLayout_2.addWidget(self.tab1_crop_box)
        self.tab1_scaling_box = QtWidgets.QGroupBox(self.frame)
        self.tab1_scaling_box.setEnabled(False)
        self.tab1_scaling_box.setObjectName("tab1_scaling_box")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.tab1_scaling_box)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.tab1_set_scaling_btn = QtWidgets.QPushButton(self.tab1_scaling_box)
        self.tab1_set_scaling_btn.setObjectName("tab1_set_scaling_btn")
        self.verticalLayout_14.addWidget(self.tab1_set_scaling_btn)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_3 = QtWidgets.QLabel(self.tab1_scaling_box)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_7.addWidget(self.label_3)
        self.tab1_set_scaling_text = QtWidgets.QLineEdit(self.tab1_scaling_box)
        self.tab1_set_scaling_text.setObjectName("tab1_set_scaling_text")
        self.horizontalLayout_7.addWidget(self.tab1_set_scaling_text)
        self.verticalLayout_14.addLayout(self.horizontalLayout_7)
        self.tab1_remove_scaling_btn = QtWidgets.QPushButton(self.tab1_scaling_box)
        self.tab1_remove_scaling_btn.setObjectName("tab1_remove_scaling_btn")
        self.verticalLayout_14.addWidget(self.tab1_remove_scaling_btn)
        self.verticalLayout_2.addWidget(self.tab1_scaling_box)
        self.tab1_time_box = QtWidgets.QGroupBox(self.frame)
        self.tab1_time_box.setEnabled(False)
        self.tab1_time_box.setObjectName("tab1_time_box")
        self.verticalLayout_21 = QtWidgets.QVBoxLayout(self.tab1_time_box)
        self.verticalLayout_21.setObjectName("verticalLayout_21")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_15 = QtWidgets.QLabel(self.tab1_time_box)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_13.addWidget(self.label_15)
        self.tab1_set_dt_text = QtWidgets.QLineEdit(self.tab1_time_box)
        self.tab1_set_dt_text.setObjectName("tab1_set_dt_text")
        self.horizontalLayout_13.addWidget(self.tab1_set_dt_text)
        self.verticalLayout_21.addLayout(self.horizontalLayout_13)
        self.tab1_dt_from_video = QtWidgets.QPushButton(self.tab1_time_box)
        self.tab1_dt_from_video.setObjectName("tab1_dt_from_video")
        self.verticalLayout_21.addWidget(self.tab1_dt_from_video)
        self.verticalLayout_2.addWidget(self.tab1_time_box)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout_3.addWidget(self.frame)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 0, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 1, 2, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 1, 0, 1, 1)
        self.mplwidgetimport = MplWidgetImport(self.tabimport)
        self.mplwidgetimport.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.mplwidgetimport.setObjectName("mplwidgetimport")
        self.gridLayout.addWidget(self.mplwidgetimport, 1, 1, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem4, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout_20 = QtWidgets.QVBoxLayout()
        self.verticalLayout_20.setContentsMargins(-1, -1, 10, -1)
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.tab1_label_frame = QtWidgets.QLabel(self.tabimport)
        self.tab1_label_frame.setMinimumSize(QtCore.QSize(40, 0))
        self.tab1_label_frame.setObjectName("tab1_label_frame")
        self.horizontalLayout_12.addWidget(self.tab1_label_frame)
        self.tab1_spinbox_frame = QtWidgets.QSpinBox(self.tabimport)
        self.tab1_spinbox_frame.setEnabled(False)
        self.tab1_spinbox_frame.setObjectName("tab1_spinbox_frame")
        self.horizontalLayout_12.addWidget(self.tab1_spinbox_frame)
        self.tab1_frameslider = QtWidgets.QSlider(self.tabimport)
        self.tab1_frameslider.setEnabled(False)
        self.tab1_frameslider.setOrientation(QtCore.Qt.Horizontal)
        self.tab1_frameslider.setObjectName("tab1_frameslider")
        self.horizontalLayout_12.addWidget(self.tab1_frameslider)
        self.verticalLayout_20.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.tab1_label_first = QtWidgets.QLabel(self.tabimport)
        self.tab1_label_first.setMinimumSize(QtCore.QSize(40, 0))
        self.tab1_label_first.setObjectName("tab1_label_first")
        self.horizontalLayout_15.addWidget(self.tab1_label_first)
        self.tab1_spinbox_first = QtWidgets.QSpinBox(self.tabimport)
        self.tab1_spinbox_first.setEnabled(False)
        self.tab1_spinbox_first.setObjectName("tab1_spinbox_first")
        self.horizontalLayout_15.addWidget(self.tab1_spinbox_first)
        self.tab1_frameslider_first = QtWidgets.QSlider(self.tabimport)
        self.tab1_frameslider_first.setEnabled(False)
        self.tab1_frameslider_first.setOrientation(QtCore.Qt.Horizontal)
        self.tab1_frameslider_first.setObjectName("tab1_frameslider_first")
        self.horizontalLayout_15.addWidget(self.tab1_frameslider_first)
        self.verticalLayout_20.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.tab1_label_last = QtWidgets.QLabel(self.tabimport)
        self.tab1_label_last.setMinimumSize(QtCore.QSize(40, 0))
        self.tab1_label_last.setObjectName("tab1_label_last")
        self.horizontalLayout_16.addWidget(self.tab1_label_last)
        self.tab1_spinbox_last = QtWidgets.QSpinBox(self.tabimport)
        self.tab1_spinbox_last.setEnabled(False)
        self.tab1_spinbox_last.setObjectName("tab1_spinbox_last")
        self.horizontalLayout_16.addWidget(self.tab1_spinbox_last)
        self.tab1_frameslider_last = QtWidgets.QSlider(self.tabimport)
        self.tab1_frameslider_last.setEnabled(False)
        self.tab1_frameslider_last.setProperty("value", 99)
        self.tab1_frameslider_last.setOrientation(QtCore.Qt.Horizontal)
        self.tab1_frameslider_last.setInvertedAppearance(False)
        self.tab1_frameslider_last.setInvertedControls(False)
        self.tab1_frameslider_last.setObjectName("tab1_frameslider_last")
        self.horizontalLayout_16.addWidget(self.tab1_frameslider_last)
        self.verticalLayout_20.addLayout(self.horizontalLayout_16)
        self.verticalLayout.addLayout(self.verticalLayout_20)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.tabWidget.addTab(self.tabimport, "")
        self.tabdetect = QtWidgets.QWidget()
        self.tabdetect.setObjectName("tabdetect")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.tabdetect)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.tab2_options = QtWidgets.QFrame(self.tabdetect)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab2_options.sizePolicy().hasHeightForWidth())
        self.tab2_options.setSizePolicy(sizePolicy)
        self.tab2_options.setMaximumSize(QtCore.QSize(150, 16777215))
        self.tab2_options.setBaseSize(QtCore.QSize(150, 0))
        self.tab2_options.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.tab2_options.setFrameShadow(QtWidgets.QFrame.Raised)
        self.tab2_options.setObjectName("tab2_options")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab2_options)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tab2_canny_box = QtWidgets.QGroupBox(self.tab2_options)
        self.tab2_canny_box.setEnabled(False)
        self.tab2_canny_box.setWhatsThis("")
        self.tab2_canny_box.setFlat(False)
        self.tab2_canny_box.setCheckable(True)
        self.tab2_canny_box.setObjectName("tab2_canny_box")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab2_canny_box)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.tab2_canny_box)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_5.addWidget(self.label_4)
        self.tab2_canny_threshold1 = QtWidgets.QSlider(self.tab2_canny_box)
        self.tab2_canny_threshold1.setMaximum(255)
        self.tab2_canny_threshold1.setProperty("value", 200)
        self.tab2_canny_threshold1.setOrientation(QtCore.Qt.Horizontal)
        self.tab2_canny_threshold1.setObjectName("tab2_canny_threshold1")
        self.verticalLayout_5.addWidget(self.tab2_canny_threshold1)
        self.label_5 = QtWidgets.QLabel(self.tab2_canny_box)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_5.addWidget(self.label_5)
        self.tab2_canny_threshold2 = QtWidgets.QSlider(self.tab2_canny_box)
        self.tab2_canny_threshold2.setMaximum(255)
        self.tab2_canny_threshold2.setProperty("value", 100)
        self.tab2_canny_threshold2.setOrientation(QtCore.Qt.Horizontal)
        self.tab2_canny_threshold2.setObjectName("tab2_canny_threshold2")
        self.verticalLayout_5.addWidget(self.tab2_canny_threshold2)
        self.label_8 = QtWidgets.QLabel(self.tab2_canny_box)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_5.addWidget(self.label_8)
        self.tab2_canny_smooth_size = QtWidgets.QSpinBox(self.tab2_canny_box)
        self.tab2_canny_smooth_size.setMaximum(50)
        self.tab2_canny_smooth_size.setProperty("value", 0)
        self.tab2_canny_smooth_size.setObjectName("tab2_canny_smooth_size")
        self.verticalLayout_5.addWidget(self.tab2_canny_smooth_size)
        self.label_9 = QtWidgets.QLabel(self.tab2_canny_box)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_5.addWidget(self.label_9)
        self.tab2_canny_dilatation_steps = QtWidgets.QSpinBox(self.tab2_canny_box)
        self.tab2_canny_dilatation_steps.setMaximum(50)
        self.tab2_canny_dilatation_steps.setProperty("value", 0)
        self.tab2_canny_dilatation_steps.setObjectName("tab2_canny_dilatation_steps")
        self.verticalLayout_5.addWidget(self.tab2_canny_dilatation_steps)
        self.verticalLayout_4.addWidget(self.tab2_canny_box)
        self.tab2_contour_box = QtWidgets.QGroupBox(self.tab2_options)
        self.tab2_contour_box.setEnabled(False)
        self.tab2_contour_box.setFlat(False)
        self.tab2_contour_box.setCheckable(True)
        self.tab2_contour_box.setChecked(False)
        self.tab2_contour_box.setObjectName("tab2_contour_box")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab2_contour_box)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_11 = QtWidgets.QLabel(self.tab2_contour_box)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_7.addWidget(self.label_11)
        self.tab2_contour_level = QtWidgets.QSlider(self.tab2_contour_box)
        self.tab2_contour_level.setMaximum(255)
        self.tab2_contour_level.setProperty("value", 128)
        self.tab2_contour_level.setOrientation(QtCore.Qt.Horizontal)
        self.tab2_contour_level.setObjectName("tab2_contour_level")
        self.verticalLayout_7.addWidget(self.tab2_contour_level)
        self.verticalLayout_4.addWidget(self.tab2_contour_box)
        self.tab2_options_box = QtWidgets.QGroupBox(self.tab2_options)
        self.tab2_options_box.setEnabled(False)
        self.tab2_options_box.setFlat(False)
        self.tab2_options_box.setCheckable(False)
        self.tab2_options_box.setObjectName("tab2_options_box")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tab2_options_box)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.tab2_options_box)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_8.addWidget(self.label_6)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.tab2_nmb_edges_1 = QtWidgets.QRadioButton(self.tab2_options_box)
        self.tab2_nmb_edges_1.setChecked(False)
        self.tab2_nmb_edges_1.setObjectName("tab2_nmb_edges_1")
        self.horizontalLayout_8.addWidget(self.tab2_nmb_edges_1)
        self.tab2_nmb_edges_2 = QtWidgets.QRadioButton(self.tab2_options_box)
        self.tab2_nmb_edges_2.setChecked(True)
        self.tab2_nmb_edges_2.setObjectName("tab2_nmb_edges_2")
        self.horizontalLayout_8.addWidget(self.tab2_nmb_edges_2)
        self.verticalLayout_8.addLayout(self.horizontalLayout_8)
        self.label_7 = QtWidgets.QLabel(self.tab2_options_box)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_8.addWidget(self.label_7)
        self.tab2_ign_pixels = QtWidgets.QSpinBox(self.tab2_options_box)
        self.tab2_ign_pixels.setMaximum(50)
        self.tab2_ign_pixels.setProperty("value", 2)
        self.tab2_ign_pixels.setObjectName("tab2_ign_pixels")
        self.verticalLayout_8.addWidget(self.tab2_ign_pixels)
        self.label_10 = QtWidgets.QLabel(self.tab2_options_box)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_8.addWidget(self.label_10)
        self.tab2_size_ratio = QtWidgets.QSlider(self.tab2_options_box)
        self.tab2_size_ratio.setMaximum(100)
        self.tab2_size_ratio.setProperty("value", 50)
        self.tab2_size_ratio.setOrientation(QtCore.Qt.Horizontal)
        self.tab2_size_ratio.setObjectName("tab2_size_ratio")
        self.verticalLayout_8.addWidget(self.tab2_size_ratio)
        self.verticalLayout_4.addWidget(self.tab2_options_box)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem5)
        self.horizontalLayout_5.addWidget(self.tab2_options)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.mplwidgetdetect = MplWidgetDetect(self.tabdetect)
        self.mplwidgetdetect.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.mplwidgetdetect.setObjectName("mplwidgetdetect")
        self.gridLayout_2.addWidget(self.mplwidgetdetect, 1, 1, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem6, 2, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem7, 1, 2, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem8, 0, 1, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 1, 0, 1, 1)
        self.verticalLayout_6.addLayout(self.gridLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.tab2_label_frame = QtWidgets.QLabel(self.tabdetect)
        self.tab2_label_frame.setObjectName("tab2_label_frame")
        self.horizontalLayout_4.addWidget(self.tab2_label_frame)
        self.tab2_spinbox = QtWidgets.QSpinBox(self.tabdetect)
        self.tab2_spinbox.setEnabled(False)
        self.tab2_spinbox.setObjectName("tab2_spinbox")
        self.horizontalLayout_4.addWidget(self.tab2_spinbox)
        self.tab2_frameslider = QtWidgets.QSlider(self.tabdetect)
        self.tab2_frameslider.setEnabled(False)
        self.tab2_frameslider.setOrientation(QtCore.Qt.Horizontal)
        self.tab2_frameslider.setObjectName("tab2_frameslider")
        self.horizontalLayout_4.addWidget(self.tab2_frameslider)
        self.verticalLayout_6.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5.addLayout(self.verticalLayout_6)
        self.tabWidget.addTab(self.tabdetect, "")
        self.tabfit = QtWidgets.QWidget()
        self.tabfit.setObjectName("tabfit")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.tabfit)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.frame_3 = QtWidgets.QFrame(self.tabfit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_3.sizePolicy().hasHeightForWidth())
        self.frame_3.setSizePolicy(sizePolicy)
        self.frame_3.setMinimumSize(QtCore.QSize(150, 0))
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.frame_3)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tab3_circle_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_circle_box.setEnabled(False)
        self.tab3_circle_box.setFlat(False)
        self.tab3_circle_box.setCheckable(True)
        self.tab3_circle_box.setChecked(False)
        self.tab3_circle_box.setObjectName("tab3_circle_box")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab3_circle_box)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_19 = QtWidgets.QLabel(self.tab3_circle_box)
        self.label_19.setObjectName("label_19")
        self.verticalLayout_10.addWidget(self.label_19)
        self.tab3_circle_ymin = QtWidgets.QSlider(self.tab3_circle_box)
        self.tab3_circle_ymin.setMaximum(999999999)
        self.tab3_circle_ymin.setProperty("value", 0)
        self.tab3_circle_ymin.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_circle_ymin.setObjectName("tab3_circle_ymin")
        self.verticalLayout_10.addWidget(self.tab3_circle_ymin)
        self.verticalLayout_9.addWidget(self.tab3_circle_box)
        self.tab3_ellipse_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_ellipse_box.setEnabled(False)
        self.tab3_ellipse_box.setFlat(False)
        self.tab3_ellipse_box.setCheckable(True)
        self.tab3_ellipse_box.setChecked(False)
        self.tab3_ellipse_box.setObjectName("tab3_ellipse_box")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.tab3_ellipse_box)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.label_21 = QtWidgets.QLabel(self.tab3_ellipse_box)
        self.label_21.setObjectName("label_21")
        self.verticalLayout_16.addWidget(self.label_21)
        self.tab3_ellipse_ymin = QtWidgets.QSlider(self.tab3_ellipse_box)
        self.tab3_ellipse_ymin.setMaximum(999999999)
        self.tab3_ellipse_ymin.setProperty("value", 0)
        self.tab3_ellipse_ymin.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_ellipse_ymin.setObjectName("tab3_ellipse_ymin")
        self.verticalLayout_16.addWidget(self.tab3_ellipse_ymin)
        self.verticalLayout_9.addWidget(self.tab3_ellipse_box)
        self.tab3_ellipses_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_ellipses_box.setEnabled(False)
        self.tab3_ellipses_box.setFlat(False)
        self.tab3_ellipses_box.setCheckable(True)
        self.tab3_ellipses_box.setChecked(True)
        self.tab3_ellipses_box.setObjectName("tab3_ellipses_box")
        self.verticalLayout_26 = QtWidgets.QVBoxLayout(self.tab3_ellipses_box)
        self.verticalLayout_26.setObjectName("verticalLayout_26")
        self.label_22 = QtWidgets.QLabel(self.tab3_ellipses_box)
        self.label_22.setObjectName("label_22")
        self.verticalLayout_26.addWidget(self.label_22)
        self.tab3_ellipses_ymin = QtWidgets.QSlider(self.tab3_ellipses_box)
        self.tab3_ellipses_ymin.setMaximum(999999999)
        self.tab3_ellipses_ymin.setProperty("value", 0)
        self.tab3_ellipses_ymin.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_ellipses_ymin.setObjectName("tab3_ellipses_ymin")
        self.verticalLayout_26.addWidget(self.tab3_ellipses_ymin)
        self.verticalLayout_9.addWidget(self.tab3_ellipses_box)
        self.tab3_polyline_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_polyline_box.setEnabled(False)
        self.tab3_polyline_box.setFlat(False)
        self.tab3_polyline_box.setCheckable(True)
        self.tab3_polyline_box.setChecked(False)
        self.tab3_polyline_box.setObjectName("tab3_polyline_box")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.tab3_polyline_box)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.label_12 = QtWidgets.QLabel(self.tab3_polyline_box)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_17.addWidget(self.label_12)
        self.tab3_polyline_deg = QtWidgets.QSpinBox(self.tab3_polyline_box)
        self.tab3_polyline_deg.setMinimum(1)
        self.tab3_polyline_deg.setMaximum(50)
        self.tab3_polyline_deg.setProperty("value", 5)
        self.tab3_polyline_deg.setObjectName("tab3_polyline_deg")
        self.verticalLayout_17.addWidget(self.tab3_polyline_deg)
        self.verticalLayout_9.addWidget(self.tab3_polyline_box)
        self.tab3_spline_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_spline_box.setEnabled(False)
        self.tab3_spline_box.setFlat(False)
        self.tab3_spline_box.setCheckable(True)
        self.tab3_spline_box.setChecked(False)
        self.tab3_spline_box.setObjectName("tab3_spline_box")
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(self.tab3_spline_box)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.label_13 = QtWidgets.QLabel(self.tab3_spline_box)
        self.label_13.setObjectName("label_13")
        self.verticalLayout_18.addWidget(self.label_13)
        self.tab3_spline_deg = QtWidgets.QSpinBox(self.tab3_spline_box)
        self.tab3_spline_deg.setMinimum(1)
        self.tab3_spline_deg.setMaximum(5)
        self.tab3_spline_deg.setProperty("value", 5)
        self.tab3_spline_deg.setObjectName("tab3_spline_deg")
        self.verticalLayout_18.addWidget(self.tab3_spline_deg)
        self.label_14 = QtWidgets.QLabel(self.tab3_spline_box)
        self.label_14.setObjectName("label_14")
        self.verticalLayout_18.addWidget(self.label_14)
        self.tab3_spline_smooth = QtWidgets.QSlider(self.tab3_spline_box)
        self.tab3_spline_smooth.setMaximum(100)
        self.tab3_spline_smooth.setProperty("value", 75)
        self.tab3_spline_smooth.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_spline_smooth.setObjectName("tab3_spline_smooth")
        self.verticalLayout_18.addWidget(self.tab3_spline_smooth)
        self.verticalLayout_9.addWidget(self.tab3_spline_box)
        self.tab3_wetting_ridge_box = QtWidgets.QGroupBox(self.frame_3)
        self.tab3_wetting_ridge_box.setEnabled(False)
        self.tab3_wetting_ridge_box.setFlat(False)
        self.tab3_wetting_ridge_box.setCheckable(True)
        self.tab3_wetting_ridge_box.setChecked(False)
        self.tab3_wetting_ridge_box.setObjectName("tab3_wetting_ridge_box")
        self.verticalLayout_29 = QtWidgets.QVBoxLayout(self.tab3_wetting_ridge_box)
        self.verticalLayout_29.setObjectName("verticalLayout_29")
        self.label_16 = QtWidgets.QLabel(self.tab3_wetting_ridge_box)
        self.label_16.setObjectName("label_16")
        self.verticalLayout_29.addWidget(self.label_16)
        self.tab3_wetting_ridge_pos_estimate = QtWidgets.QSlider(self.tab3_wetting_ridge_box)
        self.tab3_wetting_ridge_pos_estimate.setMinimum(1)
        self.tab3_wetting_ridge_pos_estimate.setMaximum(50)
        self.tab3_wetting_ridge_pos_estimate.setProperty("value", 25)
        self.tab3_wetting_ridge_pos_estimate.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_wetting_ridge_pos_estimate.setObjectName("tab3_wetting_ridge_pos_estimate")
        self.verticalLayout_29.addWidget(self.tab3_wetting_ridge_pos_estimate)
        self.label_18 = QtWidgets.QLabel(self.tab3_wetting_ridge_box)
        self.label_18.setObjectName("label_18")
        self.verticalLayout_29.addWidget(self.label_18)
        self.tab3_wetting_ridge_sigma = QtWidgets.QSlider(self.tab3_wetting_ridge_box)
        self.tab3_wetting_ridge_sigma.setMinimum(1)
        self.tab3_wetting_ridge_sigma.setMaximum(100)
        self.tab3_wetting_ridge_sigma.setProperty("value", 50)
        self.tab3_wetting_ridge_sigma.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_wetting_ridge_sigma.setObjectName("tab3_wetting_ridge_sigma")
        self.verticalLayout_29.addWidget(self.tab3_wetting_ridge_sigma)
        self.verticalLayout_9.addWidget(self.tab3_wetting_ridge_box)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem10)
        self.horizontalLayout_11.addWidget(self.frame_3)
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.mplwidgetfit = MplWidgetFit(self.tabfit)
        self.mplwidgetfit.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.mplwidgetfit.setObjectName("mplwidgetfit")
        self.gridLayout_3.addWidget(self.mplwidgetfit, 1, 1, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem11, 2, 1, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem12, 1, 2, 1, 1)
        spacerItem13 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem13, 0, 1, 1, 1)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem14, 1, 0, 1, 1)
        self.verticalLayout_15.addLayout(self.gridLayout_3)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.tab3_label_frame = QtWidgets.QLabel(self.tabfit)
        self.tab3_label_frame.setObjectName("tab3_label_frame")
        self.horizontalLayout_10.addWidget(self.tab3_label_frame)
        self.tab3_spinbox = QtWidgets.QSpinBox(self.tabfit)
        self.tab3_spinbox.setEnabled(False)
        self.tab3_spinbox.setObjectName("tab3_spinbox")
        self.horizontalLayout_10.addWidget(self.tab3_spinbox)
        self.tab3_frameslider = QtWidgets.QSlider(self.tabfit)
        self.tab3_frameslider.setEnabled(False)
        self.tab3_frameslider.setOrientation(QtCore.Qt.Horizontal)
        self.tab3_frameslider.setObjectName("tab3_frameslider")
        self.horizontalLayout_10.addWidget(self.tab3_frameslider)
        self.verticalLayout_15.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_11.addLayout(self.verticalLayout_15)
        self.tabWidget.addTab(self.tabfit, "")
        self.tabanalyze = QtWidgets.QWidget()
        self.tabanalyze.setObjectName("tabanalyze")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.tabanalyze)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.frame_4 = QtWidgets.QFrame(self.tabanalyze)
        self.frame_4.setMinimumSize(QtCore.QSize(175, 0))
        self.frame_4.setMaximumSize(QtCore.QSize(175, 16777215))
        self.frame_4.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_4.setObjectName("frame_4")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.frame_4)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.tab4_run_box = QtWidgets.QGroupBox(self.frame_4)
        self.tab4_run_box.setEnabled(False)
        self.tab4_run_box.setObjectName("tab4_run_box")
        self.verticalLayout_24 = QtWidgets.QVBoxLayout(self.tab4_run_box)
        self.verticalLayout_24.setObjectName("verticalLayout_24")
        self.label = QtWidgets.QLabel(self.tab4_run_box)
        self.label.setObjectName("label")
        self.verticalLayout_24.addWidget(self.label)
        self.tab4_set_N = QtWidgets.QSpinBox(self.tab4_run_box)
        self.tab4_set_N.setObjectName("tab4_set_N")
        self.verticalLayout_24.addWidget(self.tab4_set_N)
        self.tab4_run_button = QtWidgets.QPushButton(self.tab4_run_box)
        self.tab4_run_button.setMinimumSize(QtCore.QSize(0, 30))
        self.tab4_run_button.setObjectName("tab4_run_button")
        self.verticalLayout_24.addWidget(self.tab4_run_button)
        self.tab4_run_all_button = QtWidgets.QPushButton(self.tab4_run_box)
        self.tab4_run_all_button.setMinimumSize(QtCore.QSize(0, 30))
        self.tab4_run_all_button.setObjectName("tab4_run_all_button")
        self.verticalLayout_24.addWidget(self.tab4_run_all_button)
        self.verticalLayout_11.addWidget(self.tab4_run_box)
        spacerItem15 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem15)
        self.tab4_xaxis_box = QtWidgets.QGroupBox(self.frame_4)
        self.tab4_xaxis_box.setEnabled(False)
        self.tab4_xaxis_box.setFlat(False)
        self.tab4_xaxis_box.setCheckable(False)
        self.tab4_xaxis_box.setChecked(False)
        self.tab4_xaxis_box.setObjectName("tab4_xaxis_box")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.tab4_xaxis_box)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.tab4_combo_xaxis = QtWidgets.QComboBox(self.tab4_xaxis_box)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab4_combo_xaxis.sizePolicy().hasHeightForWidth())
        self.tab4_combo_xaxis.setSizePolicy(sizePolicy)
        self.tab4_combo_xaxis.setObjectName("tab4_combo_xaxis")
        self.verticalLayout_13.addWidget(self.tab4_combo_xaxis)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_23 = QtWidgets.QLabel(self.tab4_xaxis_box)
        self.label_23.setObjectName("label_23")
        self.horizontalLayout_14.addWidget(self.label_23)
        self.tab4_smoothx = QtWidgets.QSpinBox(self.tab4_xaxis_box)
        self.tab4_smoothx.setMaximum(1000)
        self.tab4_smoothx.setProperty("value", 0)
        self.tab4_smoothx.setObjectName("tab4_smoothx")
        self.horizontalLayout_14.addWidget(self.tab4_smoothx)
        self.verticalLayout_13.addLayout(self.horizontalLayout_14)
        self.verticalLayout_11.addWidget(self.tab4_xaxis_box)
        self.tab4_yaxis_box = QtWidgets.QGroupBox(self.frame_4)
        self.tab4_yaxis_box.setEnabled(False)
        self.tab4_yaxis_box.setFlat(False)
        self.tab4_yaxis_box.setCheckable(False)
        self.tab4_yaxis_box.setChecked(False)
        self.tab4_yaxis_box.setObjectName("tab4_yaxis_box")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout(self.tab4_yaxis_box)
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.tab4_combo_yaxis = QtWidgets.QComboBox(self.tab4_yaxis_box)
        self.tab4_combo_yaxis.setObjectName("tab4_combo_yaxis")
        self.verticalLayout_19.addWidget(self.tab4_combo_yaxis)
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.label_24 = QtWidgets.QLabel(self.tab4_yaxis_box)
        self.label_24.setObjectName("label_24")
        self.horizontalLayout_17.addWidget(self.label_24)
        self.tab4_smoothy = QtWidgets.QSpinBox(self.tab4_yaxis_box)
        self.tab4_smoothy.setMaximum(1000)
        self.tab4_smoothy.setProperty("value", 0)
        self.tab4_smoothy.setObjectName("tab4_smoothy")
        self.horizontalLayout_17.addWidget(self.tab4_smoothy)
        self.verticalLayout_19.addLayout(self.horizontalLayout_17)
        self.verticalLayout_11.addWidget(self.tab4_yaxis_box)
        self.tab4_yaxis2_box = QtWidgets.QGroupBox(self.frame_4)
        self.tab4_yaxis2_box.setEnabled(False)
        self.tab4_yaxis2_box.setFlat(False)
        self.tab4_yaxis2_box.setCheckable(True)
        self.tab4_yaxis2_box.setChecked(False)
        self.tab4_yaxis2_box.setObjectName("tab4_yaxis2_box")
        self.verticalLayout_23 = QtWidgets.QVBoxLayout(self.tab4_yaxis2_box)
        self.verticalLayout_23.setObjectName("verticalLayout_23")
        self.tab4_combo_yaxis2 = QtWidgets.QComboBox(self.tab4_yaxis2_box)
        self.tab4_combo_yaxis2.setObjectName("tab4_combo_yaxis2")
        self.verticalLayout_23.addWidget(self.tab4_combo_yaxis2)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_26 = QtWidgets.QLabel(self.tab4_yaxis2_box)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_19.addWidget(self.label_26)
        self.tab4_smoothy2 = QtWidgets.QSpinBox(self.tab4_yaxis2_box)
        self.tab4_smoothy2.setMaximum(1000)
        self.tab4_smoothy2.setProperty("value", 0)
        self.tab4_smoothy2.setObjectName("tab4_smoothy2")
        self.horizontalLayout_19.addWidget(self.tab4_smoothy2)
        self.verticalLayout_23.addLayout(self.horizontalLayout_19)
        self.verticalLayout_11.addWidget(self.tab4_yaxis2_box)
        spacerItem16 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem16)
        self.tab4_local_values_box = QtWidgets.QGroupBox(self.frame_4)
        self.tab4_local_values_box.setEnabled(False)
        self.tab4_local_values_box.setObjectName("tab4_local_values_box")
        self.verticalLayout_22 = QtWidgets.QVBoxLayout(self.tab4_local_values_box)
        self.verticalLayout_22.setObjectName("verticalLayout_22")
        self.tab4_local_x_label = QtWidgets.QLabel(self.tab4_local_values_box)
        self.tab4_local_x_label.setObjectName("tab4_local_x_label")
        self.verticalLayout_22.addWidget(self.tab4_local_x_label)
        self.tab4_local_x_value = QtWidgets.QLineEdit(self.tab4_local_values_box)
        self.tab4_local_x_value.setReadOnly(True)
        self.tab4_local_x_value.setObjectName("tab4_local_x_value")
        self.verticalLayout_22.addWidget(self.tab4_local_x_value)
        self.tab4_local_y_label = QtWidgets.QLabel(self.tab4_local_values_box)
        self.tab4_local_y_label.setObjectName("tab4_local_y_label")
        self.verticalLayout_22.addWidget(self.tab4_local_y_label)
        self.tab4_local_y_value = QtWidgets.QLineEdit(self.tab4_local_values_box)
        self.tab4_local_y_value.setReadOnly(True)
        self.tab4_local_y_value.setObjectName("tab4_local_y_value")
        self.verticalLayout_22.addWidget(self.tab4_local_y_value)
        self.tab4_local_y2_label = QtWidgets.QLabel(self.tab4_local_values_box)
        self.tab4_local_y2_label.setObjectName("tab4_local_y2_label")
        self.verticalLayout_22.addWidget(self.tab4_local_y2_label)
        self.tab4_local_y2_value = QtWidgets.QLineEdit(self.tab4_local_values_box)
        self.tab4_local_y2_value.setReadOnly(True)
        self.tab4_local_y2_value.setObjectName("tab4_local_y2_value")
        self.verticalLayout_22.addWidget(self.tab4_local_y2_value)
        self.verticalLayout_11.addWidget(self.tab4_local_values_box)
        spacerItem17 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem17)
        self.horizontalLayout_9.addWidget(self.frame_4)
        self.verticalLayout_25 = QtWidgets.QVBoxLayout()
        self.verticalLayout_25.setContentsMargins(-1, -1, 0, 0)
        self.verticalLayout_25.setObjectName("verticalLayout_25")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem18 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem18, 0, 1, 1, 1)
        spacerItem19 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem19, 1, 0, 1, 1)
        spacerItem20 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem20, 1, 2, 1, 1)
        spacerItem21 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem21, 2, 1, 1, 1)
        self.mplwidgetanalyze = MplWidgetAnalyze(self.tabanalyze)
        self.mplwidgetanalyze.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.mplwidgetanalyze.setObjectName("mplwidgetanalyze")
        self.gridLayout_4.addWidget(self.mplwidgetanalyze, 1, 1, 1, 1)
        self.verticalLayout_25.addLayout(self.gridLayout_4)
        self.horizontalLayout_9.addLayout(self.verticalLayout_25)
        self.tabWidget.addTab(self.tabanalyze, "")
        self.tabdata = QtWidgets.QWidget()
        self.tabdata.setEnabled(False)
        self.tabdata.setObjectName("tabdata")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tabdata)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.frame_5 = QtWidgets.QFrame(self.tabdata)
        self.frame_5.setMinimumSize(QtCore.QSize(175, 0))
        self.frame_5.setMaximumSize(QtCore.QSize(175, 16777215))
        self.frame_5.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_5.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_5.setObjectName("frame_5")
        self.verticalLayout_28 = QtWidgets.QVBoxLayout(self.frame_5)
        self.verticalLayout_28.setObjectName("verticalLayout_28")
        self.tab5_export_box = QtWidgets.QGroupBox(self.frame_5)
        self.tab5_export_box.setEnabled(False)
        self.tab5_export_box.setFlat(False)
        self.tab5_export_box.setCheckable(False)
        self.tab5_export_box.setChecked(False)
        self.tab5_export_box.setObjectName("tab5_export_box")
        self.verticalLayout_30 = QtWidgets.QVBoxLayout(self.tab5_export_box)
        self.verticalLayout_30.setObjectName("verticalLayout_30")
        self.tab5_export_csv_button = QtWidgets.QPushButton(self.tab5_export_box)
        self.tab5_export_csv_button.setObjectName("tab5_export_csv_button")
        self.verticalLayout_30.addWidget(self.tab5_export_csv_button)
        self.tab5_export_xlsx_button = QtWidgets.QPushButton(self.tab5_export_box)
        self.tab5_export_xlsx_button.setObjectName("tab5_export_xlsx_button")
        self.verticalLayout_30.addWidget(self.tab5_export_xlsx_button)
        self.verticalLayout_28.addWidget(self.tab5_export_box)
        self.tab5_number_format_box = QtWidgets.QGroupBox(self.frame_5)
        self.tab5_number_format_box.setEnabled(False)
        self.tab5_number_format_box.setFlat(False)
        self.tab5_number_format_box.setCheckable(False)
        self.tab5_number_format_box.setChecked(False)
        self.tab5_number_format_box.setObjectName("tab5_number_format_box")
        self.verticalLayout_27 = QtWidgets.QVBoxLayout(self.tab5_number_format_box)
        self.verticalLayout_27.setObjectName("verticalLayout_27")
        self.label_2 = QtWidgets.QLabel(self.tab5_number_format_box)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_27.addWidget(self.label_2)
        self.tab5_significativ_numbers = QtWidgets.QSpinBox(self.tab5_number_format_box)
        self.tab5_significativ_numbers.setMaximum(1000)
        self.tab5_significativ_numbers.setProperty("value", 3)
        self.tab5_significativ_numbers.setObjectName("tab5_significativ_numbers")
        self.verticalLayout_27.addWidget(self.tab5_significativ_numbers)
        self.verticalLayout_28.addWidget(self.tab5_number_format_box)
        spacerItem22 = QtWidgets.QSpacerItem(20, 562, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_28.addItem(spacerItem22)
        self.horizontalLayout.addWidget(self.frame_5)
        self.tab5_DataTable = QtWidgets.QTableWidget(self.tabdata)
        self.tab5_DataTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tab5_DataTable.setDragEnabled(True)
        self.tab5_DataTable.setDragDropMode(QtWidgets.QAbstractItemView.DragOnly)
        self.tab5_DataTable.setAlternatingRowColors(True)
        self.tab5_DataTable.setObjectName("tab5_DataTable")
        self.tab5_DataTable.setColumnCount(0)
        self.tab5_DataTable.setRowCount(0)
        self.tab5_DataTable.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.tab5_DataTable)
        self.tabWidget.addTab(self.tabdata, "")
        self.tab4_log = QtWidgets.QWidget()
        self.tab4_log.setObjectName("tab4_log")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.tab4_log)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.logarea = QtWidgets.QTextEdit(self.tab4_log)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.logarea.setFont(font)
        self.logarea.setUndoRedoEnabled(False)
        self.logarea.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        self.logarea.setReadOnly(True)
        self.logarea.setObjectName("logarea")
        self.horizontalLayout_6.addWidget(self.logarea)
        self.tabWidget.addTab(self.tab4_log, "")
        self.horizontalLayout_2.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 972, 20))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(4)
        self.tab1_open_button.clicked.connect(MainWindow.tab1.import_file)
        self.tab1_reset_crop.clicked.connect(MainWindow.tab1.reset_crop)
        self.tabWidget.currentChanged['int'].connect(MainWindow.change_tab)
        self.tab2_spinbox.valueChanged['int'].connect(MainWindow.tab2.set_current_frame)
        self.tab2_frameslider.valueChanged['int'].connect(self.tab2_spinbox.setValue)
        self.tab2_spinbox.valueChanged['int'].connect(self.tab2_frameslider.setValue)
        self.tab1_set_scaling_btn.clicked.connect(MainWindow.tab1.set_scaling)
        self.tab1_remove_scaling_btn.clicked.connect(MainWindow.tab1.remove_scaling)
        self.tab2_canny_smooth_size.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_canny_dilatation_steps.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_contour_level.sliderMoved['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_ign_pixels.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_size_ratio.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_nmb_edges_2.toggled['bool'].connect(MainWindow.tab2.update_edge)
        self.tab2_nmb_edges_1.toggled['bool'].connect(MainWindow.tab2.update_edge)
        self.tab2_contour_box.toggled['bool'].connect(MainWindow.tab2.toggle_contour)
        self.tab2_canny_box.toggled['bool'].connect(MainWindow.tab2.toggle_canny)
        self.tab3_spinbox.valueChanged['int'].connect(self.tab3_frameslider.setValue)
        self.tab3_spinbox.valueChanged['int'].connect(MainWindow.tab3.set_current_frame)
        self.tab3_frameslider.valueChanged['int'].connect(self.tab3_spinbox.setValue)
        self.tab3_spline_smooth.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_spline_box.toggled['bool'].connect(MainWindow.tab3.toggle_spline)
        self.tab3_spline_deg.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_polyline_box.toggled['bool'].connect(MainWindow.tab3.toggle_polyline)
        self.tab3_polyline_deg.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_ellipse_box.toggled['bool'].connect(MainWindow.tab3.toggle_ellipse)
        self.tab3_ellipse_ymin.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_circle_ymin.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_circle_box.toggled['bool'].connect(MainWindow.tab3.toggle_circle)
        self.tab4_combo_xaxis.currentIndexChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab4_combo_yaxis.currentIndexChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab1_spinbox_frame.valueChanged['int'].connect(MainWindow.tab1.set_current_frame)
        self.tab1_frameslider.sliderMoved['int'].connect(self.tab1_spinbox_frame.setValue)
        self.tab1_spinbox_frame.valueChanged['int'].connect(self.tab1_frameslider.setValue)
        self.tab1_frameslider_first.sliderMoved['int'].connect(self.tab1_spinbox_first.setValue)
        self.tab1_spinbox_first.valueChanged['int'].connect(self.tab1_frameslider_first.setValue)
        self.tab1_frameslider_last.sliderMoved['int'].connect(self.tab1_spinbox_last.setValue)
        self.tab1_frameslider_last.valueChanged['int'].connect(self.tab1_spinbox_last.setValue)
        self.tab1_spinbox_first.valueChanged['int'].connect(MainWindow.tab1.set_first_frame)
        self.tab1_spinbox_last.valueChanged['int'].connect(MainWindow.tab1.set_last_frame)
        self.tab1_spinbox_last.valueChanged['int'].connect(self.tab1_frameslider_last.setValue)
        self.tab2_canny_threshold1.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab2_canny_threshold2.valueChanged['int'].connect(MainWindow.tab2.update_edge)
        self.tab4_combo_yaxis2.currentIndexChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab4_yaxis2_box.toggled['bool'].connect(MainWindow.tab4.toggle_axis2)
        self.tab4_smoothx.valueChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab4_smoothy2.valueChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab4_smoothy.valueChanged['int'].connect(MainWindow.tab4.update_plot)
        self.tab4_run_button.clicked.connect(MainWindow.tab4.compute)
        self.tab3_ellipses_box.toggled['bool'].connect(MainWindow.tab3.toggle_ellipses)
        self.tab3_ellipses_ymin.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab1_dt_from_video.clicked['bool'].connect(MainWindow.tab1.dt_from_video)
        self.tab5_export_csv_button.clicked.connect(MainWindow.tab5.export_as_csv)
        self.tab5_significativ_numbers.valueChanged['int'].connect(MainWindow.tab5.update_data_table)
        self.tab3_wetting_ridge_box.toggled['bool'].connect(MainWindow.tab3.toggle_wetting_ridge)
        self.tab3_wetting_ridge_sigma.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab3_wetting_ridge_pos_estimate.valueChanged['int'].connect(MainWindow.tab3.update_fit)
        self.tab1_zoom_to_area.clicked['bool'].connect(MainWindow.tab1.zoom_to_area)
        self.tab4_run_all_button.clicked.connect(MainWindow.tab4.compute_all)
        self.tab5_export_xlsx_button.clicked['bool'].connect(MainWindow.tab5.export_as_xlsx)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Import"))
        self.tab1_open_button.setToolTip(_translate("MainWindow", "Import a file (image, video or image set)"))
        self.tab1_open_button.setText(_translate("MainWindow", "Open"))
        self.tab1_crop_box.setTitle(_translate("MainWindow", "Crop"))
        self.tab1_zoom_to_area.setToolTip(_translate("MainWindow", "Reset the cropping window"))
        self.tab1_zoom_to_area.setText(_translate("MainWindow", "Zoom to area"))
        self.tab1_reset_crop.setToolTip(_translate("MainWindow", "Reset the cropping window"))
        self.tab1_reset_crop.setText(_translate("MainWindow", "Reset"))
        self.tab1_scaling_box.setTitle(_translate("MainWindow", "Scale"))
        self.tab1_set_scaling_btn.setToolTip(_translate("MainWindow", "Allow to select two scaling points on the current image"))
        self.tab1_set_scaling_btn.setText(_translate("MainWindow", "Set scaling"))
        self.label_3.setText(_translate("MainWindow", "dl:"))
        self.tab1_set_scaling_text.setToolTip(_translate("MainWindow", "Length separating the scaling points"))
        self.tab1_set_scaling_text.setText(_translate("MainWindow", "1mm"))
        self.tab1_remove_scaling_btn.setToolTip(_translate("MainWindow", "Remove the current scaling"))
        self.tab1_remove_scaling_btn.setText(_translate("MainWindow", "Remove"))
        self.tab1_time_box.setTitle(_translate("MainWindow", "Time [s]"))
        self.label_15.setText(_translate("MainWindow", "dt:"))
        self.tab1_set_dt_text.setToolTip(_translate("MainWindow", "Interval of time between two frames (in second)"))
        self.tab1_set_dt_text.setText(_translate("MainWindow", "1"))
        self.tab1_dt_from_video.setToolTip(_translate("MainWindow", "Reset the cropping window"))
        self.tab1_dt_from_video.setText(_translate("MainWindow", "From video"))
        self.tab1_label_frame.setText(_translate("MainWindow", "Frame"))
        self.tab1_spinbox_frame.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tab1_frameslider.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tab1_label_first.setText(_translate("MainWindow", "First"))
        self.tab1_spinbox_first.setToolTip(_translate("MainWindow", "First frame to analyze"))
        self.tab1_frameslider_first.setToolTip(_translate("MainWindow", "First frame to analyze"))
        self.tab1_label_last.setText(_translate("MainWindow", "Last"))
        self.tab1_spinbox_last.setToolTip(_translate("MainWindow", "Last frame to analyze"))
        self.tab1_frameslider_last.setToolTip(_translate("MainWindow", "Last frame to analyze"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabimport), _translate("MainWindow", "Import"))
        self.tab2_canny_box.setTitle(_translate("MainWindow", "Cann&y edges"))
        self.label_4.setText(_translate("MainWindow", "Upper threshold"))
        self.tab2_canny_threshold1.setToolTip(_translate("MainWindow", "Canny edge upper threshold (lower this to detect more edges)"))
        self.label_5.setText(_translate("MainWindow", "Lower threshold"))
        self.tab2_canny_threshold2.setToolTip(_translate("MainWindow", "Canny edge lower threshold (lower this to detect longer edges)"))
        self.label_8.setToolTip(_translate("MainWindow", "Smooth actor performed before "))
        self.label_8.setText(_translate("MainWindow", "Smoothing factor"))
        self.tab2_canny_smooth_size.setToolTip(_translate("MainWindow", "Smooth the image before edge detection (increase this when the edge detection perform badly and the images are noisy)"))
        self.label_9.setText(_translate("MainWindow", "Dilatation steps"))
        self.tab2_canny_dilatation_steps.setToolTip(_translate("MainWindow", "Dilate the detected edges (increase this if the edges are discontinuous in some points)"))
        self.tab2_contour_box.setTitle(_translate("MainWindow", "Conto&ur"))
        self.label_11.setText(_translate("MainWindow", "Level"))
        self.tab2_contour_level.setToolTip(_translate("MainWindow", "Level of gray of the isocontour (to adjust depending on the edge to detect)"))
        self.tab2_options_box.setTitle(_translate("MainWindow", "Generic options"))
        self.label_6.setText(_translate("MainWindow", "Number of edges"))
        self.tab2_nmb_edges_1.setText(_translate("MainWindow", "&1"))
        self.tab2_nmb_edges_2.setToolTip(_translate("MainWindow", "Number of separated edges to look for (set this to 1 if something else than the drop edge is detected)"))
        self.tab2_nmb_edges_2.setText(_translate("MainWindow", "&2"))
        self.label_7.setText(_translate("MainWindow", "Ignored pixels"))
        self.tab2_ign_pixels.setToolTip(_translate("MainWindow", "Number of pixel ignored near the baseline (increase this if the drop edges are difficult to see near the baseline)"))
        self.label_10.setText(_translate("MainWindow", "Size ratio"))
        self.tab2_size_ratio.setToolTip(_translate("MainWindow", "Maximal size ratio between separated edges (if applicable)"))
        self.tab2_label_frame.setText(_translate("MainWindow", "Frames"))
        self.tab2_spinbox.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tab2_frameslider.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabdetect), _translate("MainWindow", "Edge detection"))
        self.tab3_circle_box.setToolTip(_translate("MainWindow", "Fit a circle to the droplet edges."))
        self.tab3_circle_box.setTitle(_translate("MainWindow", "Circle"))
        self.label_19.setText(_translate("MainWindow", "Ignore lower part"))
        self.tab3_circle_ymin.setToolTip(_translate("MainWindow", "Ignore the lower part of the drop"))
        self.tab3_ellipse_box.setToolTip(_translate("MainWindow", "Fit a single ellipse to the drop edges."))
        self.tab3_ellipse_box.setTitle(_translate("MainWindow", "Ellipse"))
        self.label_21.setText(_translate("MainWindow", "Ignore lower part"))
        self.tab3_ellipse_ymin.setToolTip(_translate("MainWindow", "Ignore the lower part of the drop"))
        self.tab3_ellipses_box.setToolTip(_translate("MainWindow", "Fit two ellipses to each droplet side."))
        self.tab3_ellipses_box.setTitle(_translate("MainWindow", "Ellipses"))
        self.label_22.setText(_translate("MainWindow", "Ignore lower part"))
        self.tab3_ellipses_ymin.setToolTip(_translate("MainWindow", "Ignore the lower part of the drop"))
        self.tab3_polyline_box.setToolTip(_translate("MainWindow", "Fit two polynomials to each droplet side."))
        self.tab3_polyline_box.setTitle(_translate("MainWindow", "Pol&ylines"))
        self.label_12.setText(_translate("MainWindow", "Degree"))
        self.tab3_polyline_deg.setToolTip(_translate("MainWindow", "Degree of the polynomial (higher means better fit, but more sensitive to noise)"))
        self.tab3_spline_box.setToolTip(_translate("MainWindow", "Fit two splines to each droplet side."))
        self.tab3_spline_box.setTitle(_translate("MainWindow", "Splines"))
        self.label_13.setText(_translate("MainWindow", "Degree"))
        self.tab3_spline_deg.setToolTip(_translate("MainWindow", "Degree of the spline (up to 5)"))
        self.label_14.setText(_translate("MainWindow", "Smooth"))
        self.tab3_spline_smooth.setToolTip(_translate("MainWindow", "Smoothiness of the spline"))
        self.tab3_wetting_ridge_box.setToolTip(_translate("MainWindow", "Fit two splines to each droplet side."))
        self.tab3_wetting_ridge_box.setTitle(_translate("MainWindow", "Wetting ridge"))
        self.label_16.setText(_translate("MainWindow", "Position estimate"))
        self.tab3_wetting_ridge_pos_estimate.setToolTip(_translate("MainWindow", "Smoothiness of the spline"))
        self.label_18.setText(_translate("MainWindow", "Sigma"))
        self.tab3_wetting_ridge_sigma.setToolTip(_translate("MainWindow", "Smoothiness of the spline"))
        self.tab3_label_frame.setText(_translate("MainWindow", "Frames"))
        self.tab3_spinbox.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tab3_frameslider.setToolTip(_translate("MainWindow", "Frame number to display"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabfit), _translate("MainWindow", "Fitting"))
        self.label.setText(_translate("MainWindow", "Use every N frames"))
        self.tab4_set_N.setToolTip(_translate("MainWindow", "The default value is calculated so that around 50 frames are analysed."))
        self.tab4_run_button.setText(_translate("MainWindow", "Compute"))
        self.tab4_run_all_button.setText(_translate("MainWindow", "Compute all"))
        self.tab4_xaxis_box.setTitle(_translate("MainWindow", "X axis"))
        self.label_23.setText(_translate("MainWindow", "Smooth"))
        self.tab4_yaxis_box.setTitle(_translate("MainWindow", "Y axis"))
        self.label_24.setText(_translate("MainWindow", "Smooth"))
        self.tab4_yaxis2_box.setTitle(_translate("MainWindow", "Y axis &2"))
        self.label_26.setText(_translate("MainWindow", "Smooth"))
        self.tab4_local_values_box.setTitle(_translate("MainWindow", "Cursor values"))
        self.tab4_local_x_label.setText(_translate("MainWindow", "x"))
        self.tab4_local_y_label.setText(_translate("MainWindow", "y"))
        self.tab4_local_y2_label.setText(_translate("MainWindow", "y2"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabanalyze), _translate("MainWindow", "Analyze"))
        self.tab5_export_box.setTitle(_translate("MainWindow", "Export"))
        self.tab5_export_csv_button.setToolTip(_translate("MainWindow", "Export all the computed data to a csv text file"))
        self.tab5_export_csv_button.setText(_translate("MainWindow", "Export as csv"))
        self.tab5_export_xlsx_button.setToolTip(_translate("MainWindow", "Export all the computed data to a csv text file"))
        self.tab5_export_xlsx_button.setText(_translate("MainWindow", "Export as xlsx"))
        self.tab5_number_format_box.setTitle(_translate("MainWindow", "Number format"))
        self.label_2.setText(_translate("MainWindow", " Significant digits"))
        self.tab5_significativ_numbers.setToolTip(_translate("MainWindow", "Only affects the datatable display"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabdata), _translate("MainWindow", "Data Table"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab4_log), _translate("MainWindow", "Log"))


from .MPLWidget import MplWidgetAnalyze, MplWidgetDetect, MplWidgetFit, MplWidgetImport



# -*- coding: utf-8 -*-
#!/usr/env python3

# Copyright (C) 2018-2019 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2018-2019"
__credits__ = ""
__license__ = "GPL3"
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


import matplotlib as mpl
import numpy as np
from pyDSA_core.baseline import Baseline


# Matplotlib 2.2 backward compatibility monkey patching
def set_center(self, xy):
    self.center = xy
    self.xy = xy
mpl.patches.Circle.set_center = set_center


class Handler(object):
    def __init__(self, canvas, fig, ax, color, hand_ratio):
        self.fig = fig
        self.ax = ax
        self.canvas = canvas
        self.background = None
        self.color = color
        self.line = None
        self.create_line()
        self.hands = []
        self.minihands = []
        self.hand_ratio = hand_ratio
        self.hand_size = 0
        self.create_hands()
        for hand in self.hands + self.minihands:
            self.ax.add_patch(hand)
        self.dragged_hand = None
        self.dragged_offset = [0, 0]
        self.dragged_ind = None

    def create_line(self):
        self.line = self.ax.plot([0], [0], color=self.color, lw=.5)[0]

    def create_hands(self):
        raise Exception('Need to be defined !')

    def update_line(self):
        raise Exception('Need to be defined !')

    def update_hands(self):
        raise Exception('Need to be defined !')

    def update_from_event(self):
        raise Exception('Need to be defined !')

    def update_hand_size(self):
        sizex = abs(self.ax.viewLim.width)
        sizey = abs(self.ax.viewLim.height)
        self.hand_size = (self.hand_ratio*sizex*sizey)**.5
        if self.hand_size > np.min([sizex, sizey])/2:
            self.hand_size = int(np.min([sizex, sizey])/2) - 1
        for hand in self.hands:
            try:
                hand.set_width(self.hand_size)
                hand.set_height(self.hand_size)
            except:
                hand.set_radius(self.hand_size/2)
        for hand in self.minihands:
            try:
                hand.set_width(self.hand_size/10)
                hand.set_height(self.hand_size/10)
            except:
                hand.set_radius(self.hand_size/20)

    def select_hand_at_point(self, event):
        ind_sel = np.argwhere([hand.contains(event)[0]
                               for hand in self.hands])
        ind_sel = ind_sel.flatten()
        if len(ind_sel) == 0:
            return False
        ind_sel = ind_sel[0]
        self.dragged_ind = ind_sel
        self.dragged_hand = self.hands[ind_sel]
        try:
            pos = self.dragged_hand.xy
        except AttributeError:
            pos = self.dragged_hand.get_center()
        self.dragged_offset = [event.xdata - pos[0],
                               event.ydata - pos[1]]
        return True

    def prepare_for_drag(self):
        for hand in self.hands + self.minihands:
            hand.set_animated(True)
        self.line.set_animated(True)
        self.canvas.draw()
        self.background = self.canvas.copy_from_bbox(self.ax.bbox)
        for hand in self.hands + self.minihands:
            self.ax.draw_artist(hand)
        self.ax.draw_artist(self.line)
        self.canvas.blit(self.ax.bbox)

    def drag_to(self, event, blit=True):
        if self.dragged_hand is None:
            return None
        # Update hands
        self.update_from_event(event)
        # Update line
        self.update_line()
        # Update handlers
        self.update_hands()
        # Redraw
        self.canvas.restore_region(self.background)
        for hand in self.hands + self.minihands:
            self.ax.draw_artist(hand)
        self.ax.draw_artist(self.line)
        # blit just the redrawn area
        if blit:
            self.canvas.blit(self.ax.bbox)

    def unselect_hand(self):
        self.dragged_hand = None
        self.dragged_ind = None
        self.dragged_offset = None

    def finish_drag(self):
        for hand in self.hands + self.minihands:
            hand.set_animated(False)
        self.line.set_animated(False)
        self.background = None

    def is_dragging(self):
        if self.dragged_hand is not None:
            return True
        else:
            return False


class RectangleHandler(Handler):
    def __init__(self, canvas, fig, ax):
        super().__init__(canvas, fig, ax, color='r', hand_ratio=1/500)
        self.lims = [[0, 0], [0, 0]]

    def create_hands(self):
        self.hands = [mpl.patches.Rectangle([0, 0],
                                            self.hand_size, self.hand_size,
                                            color=self.color,
                                            alpha=0.25)
                      for i in range(4)]
        ind_xs = [0, 1, 1, 0]
        ind_ys = [0, 0, 1, 1]
        for hand, ind_xlim, ind_ylim in zip(self.hands, ind_xs, ind_ys):
            hand.ind_xlim = ind_xlim
            hand.ind_ylim = ind_ylim

    def update_line(self):
        (xlim1, xlim2), (ylim1, ylim2) = self.lims
        xs = [xlim1, xlim2, xlim2, xlim1, xlim1]
        ys = [ylim1, ylim1, ylim2, ylim2, ylim1]
        self.line.set_data([xs, ys])

    def update_hands(self):
        size = self.hand_size
        (xlim1, xlim2), (ylim1, ylim2) = self.lims
        xs = [xlim1, xlim2 - size, xlim2 - size, xlim1]
        ys = [ylim1, ylim1, ylim2 - size, ylim2 - size]
        for hand, xlim, ylim in zip(self.hands, xs, ys):
            hand.set_xy([xlim, ylim])

    def update_from_event(self, event):
        new_xlim = event.xdata - self.dragged_offset[0]
        new_xlim += self.dragged_hand.ind_xlim*self.hand_size
        new_ylim = event.ydata - self.dragged_offset[1]
        new_ylim += self.dragged_hand.ind_ylim*self.hand_size
        self.lims[0][self.dragged_hand.ind_xlim] = new_xlim
        self.lims[1][self.dragged_hand.ind_ylim] = new_ylim
        self.ensure_lims_in_image()

    def update_lims(self, xlim1, xlim2, ylim1, ylim2):
        # update
        self.lims = [[xlim1, xlim2], [ylim1, ylim2]]
        self.ensure_lims_in_image()
        self.update_hand_size()
        self.update_line()
        self.update_hands()

    def ensure_lims_in_image(self):
        sizex = abs(self.ax.dataLim.width)
        sizey = abs(self.ax.dataLim.height)
        (xlim1, xlim2), (ylim1, ylim2) = self.lims
        # checks
        #    x
        if xlim1 < 0:
            xlim1 = 0
        if xlim2 > sizex:
            xlim2 = sizex
        if xlim2 < xlim1:
            xlim1 = xlim2
        #    y
        if ylim1 < 0:
            ylim1 = 0
        if ylim2 > sizey:
            ylim2 = sizey
        if ylim2 < ylim1:
            ylim1 = ylim2
        # Update
        self.lims = [[xlim1, xlim2], [ylim1, ylim2]]


class BaselineHandler(Handler):
    def __init__(self, canvas, fig, ax):
        super().__init__(canvas, fig, ax, color='b', hand_ratio=1/500)
        self.pt1 = None
        self.pt2 = None

    def create_hands(self):
        self.hands = [mpl.patches.Circle([0, 0],
                                         radius=self.hand_size/2,
                                         color=self.color,
                                         alpha=0.25)
                      for i in range(2)]

    def update_line(self):
        sizex = abs(self.ax.dataLim.width)
        pt1, pt2 = Baseline.get_baseline_from_points([self.pt1, self.pt2],
                                                     xmin=0,
                                                     xmax=sizex)
        self.line.set_data([pt1[0], pt2[0]], [pt1[1], pt2[1]])

    def update_hands(self):
        self.hands[0].set_center(self.pt1)
        self.hands[1].set_center(self.pt2)

    def update_from_event(self, event):
        new_x = event.xdata - self.dragged_offset[0]
        new_y = event.ydata - self.dragged_offset[1]
        if self.dragged_ind == 0:
            self.pt1 = [new_x, new_y]
        else:
            self.pt2 = [new_x, new_y]

    def update_pts(self, pt1, pt2):
        # update
        self.pt1 = pt1
        self.pt2 = pt2
        self.update_hand_size()
        self.update_line()
        self.update_hands()


class ScalingHandler(Handler):
    def __init__(self, canvas, fig, ax):
        super().__init__(canvas, fig, ax, color='g', hand_ratio=1/500)
        self.pts = []

    def create_hands(self):
        for i in range(2):
            self.hands.append(mpl.patches.Circle([-1000, -1000],
                                                 radius=self.hand_size/2,
                                                 color=self.color,
                                                 alpha=0.25))
            self.minihands.append(mpl.patches.Circle([-1000, -1000],
                                                     radius=self.hand_size/20,
                                                     color=self.color,
                                                     alpha=1))

    def update_line(self):
        pass

    def update_hands(self):
        self.hands[0].set_center(self.pts[0])
        self.hands[1].set_center(self.pts[1])
        self.minihands[0].set_center(self.pts[0])
        self.minihands[1].set_center(self.pts[1])

    def update_from_event(self, event):
        new_x = event.xdata - self.dragged_offset[0]
        new_y = event.ydata - self.dragged_offset[1]
        self.pts[self.dragged_ind] = [new_x, new_y]

    def add_point(self, event):
        self.update_hand_size()
        pos = (event.xdata, event.ydata)
        if len(self.pts) == 0:
            hand = self.hands[0]
            minihand = self.minihands[0]
        elif len(self.pts) == 1:
            hand = self.hands[1]
            minihand = self.minihands[1]
        else:
            return None
        hand.set_center(pos)
        minihand.set_center(pos)
        self.pts.append(pos)

    def reset(self):
        self.pts = []
        for hand in self.hands + self.minihands:
            hand.set_center([-1000, -1000])
        self.canvas.draw()

    def get_scale(self):
        if len(self.pts) != 2:
            return None
        return ((self.pts[0][0] - self.pts[1][0])**2
                + (self.pts[0][1] - self.pts[1][1])**2)**.5

    def update_pts(self, pts):
        # update
        self.pts = pts
        self.update_hand_size()
        self.update_hands()


class VerticalLineHandler(Handler):
    def __init__(self, canvas, fig, ax, color):
        self.pt = [0, 0]
        super().__init__(canvas, fig, ax, color=color, hand_ratio=1/500)

    def create_line(self):
        self.line = self.ax.axvline(self.pt[0], color=self.color, lw=.5)

    def create_hands(self):
        self.update_hand_size()
        self.hands.append(mpl.patches.Circle(self.pt,
                                             radius=self.hand_size,
                                             color=self.color,
                                             alpha=0.25))

    def update_hand_size(self):
        self.hand_size = 2

    def update_line(self):
        ylims = self.ax.get_ylim()
        pt1 = [self.pt[0], ylims[0]]
        pt2 = [self.pt[0], ylims[1]]
        self.line.set_data([pt1[0], pt2[0]], [0, 1])

    def update_hands(self):
        self.hands[0].set_center(self.pt)
        xlims = self.ax.get_xlim()
        self.hands[0].set_radius((xlims[1] - xlims[0])*self.hand_size/100)

    def update_from_event(self, event):
        new_x = event.xdata
        new_y = event.ydata
        self.pt = [new_x, new_y]
        self.update_hands()
        self.update_upstream()

    def reset(self):
        self.pt = [0, 0]
        self.hands[0].set_center([-1000, -1000])
        self.canvas.draw()

    def update_line_pos(self, x):
        # update
        if x is None:
            return None
        if np.isnan(x):
            return None
        self.pt[0] = x
        self.pt[1] = np.mean(self.ax.get_ylim())
        self.update_hands()
        self.update_line()
        self.update_upstream()

    def update_upstream(self):
        """ Update the interface to reflect line modification"""
        # Get values at position
        self.canvas.parent.update_upstream()

    def prepare_for_drag(self):
        self.canvas.parent.indicator1.set_animated(True)
        self.canvas.parent.indicator2.set_animated(True)
        super(VerticalLineHandler, self).prepare_for_drag()
        self.canvas.parent.ax.draw_artist(self.canvas.parent.indicator1)
        self.canvas.parent.ax2.draw_artist(self.canvas.parent.indicator2)

    def drag_to(self, event):
        super(VerticalLineHandler, self).drag_to(event, blit=False)
        self.canvas.parent.ax.draw_artist(self.canvas.parent.indicator1)
        self.canvas.parent.ax2.draw_artist(self.canvas.parent.indicator2)
        # blit just the redrawn area
        self.canvas.blit(self.ax.bbox)

    def finish_drag(self):
        super(VerticalLineHandler, self).finish_drag()
        self.canvas.parent.indicator1.set_animated(False)
        self.canvas.parent.indicator2.set_animated(False)



#!/usr/env python3

# Copyright (C) 2018-2019 Gaby Launay

# Author: Gaby Launay  <gaby.launay@tutanota.com>

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

__author__ = "Gaby Launay"
__copyright__ = "Gaby Launay 2018-2019"
__credits__ = ""
__license__ = "GPL3"
__email__ = "gaby.launay@tutanota.com"
__status__ = "Development"


import sys
import os
import argparse
from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5 import QtWidgets, QtCore, QtGui

from .design import Ui_MainWindow
from .dsa_backend import DSA_hdd as DSA
from .log import Log
from .tab import Tab
from .tabimport import TabImport
from .tabedges import TabEdges
from .tabfits import TabFits
from .tabanalyze import TabAnalyze
from .tabdata import TabData


class AppWindow(QMainWindow):
    def __init__(self, app):
        super().__init__()
        self.ui = Ui_MainWindow()
        self.globalapp = app
        # Properly set the icon
        script_path = os.path.dirname(os.path.realpath(__file__))
        icon_path = os.path.join(script_path, "../icon/pyDSA_logo.eps")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(icon_path),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)
        # Variables
        self.current_ind = 0
        self.statusbar_delay = 2000
        # Initialize log
        self.ui.logarea = None  # needed to be initialized for log
        self.ui.status_bar = None
        self.log = Log(self.ui, self.statusbar_delay)
        # Initialize dsa backend
        self.dsa = DSA(self)
        # Initialize tabs (need to be done before initializing design.py)
        self.tab1 = TabImport(self.ui, self, self.dsa, self.log)
        self.tab2 = TabEdges(self.ui, self, self.dsa, self.log)
        self.tab3 = TabFits(self.ui, self, self.dsa, self.log)
        self.tab4 = TabAnalyze(self.ui, self, self.dsa, self.log)
        self.tab5 = TabData(self.ui, self, self.dsa, self.log)
        self.tab6 = Tab(self.ui, self, self.dsa, self.log)
        self.tabs = [self.tab1, self.tab2, self.tab3, self.tab4, self.tab5,
                     self.tab6]
        self.last_tab = 0
        # Including design.ui
        self.ui.setupUi(self)
        # Add missing links
        self.ui.mplwidgetanalyze.ui = self.ui
        self.ui.mplwidgetanalyze.app = self
        # Initialize progressbar
        self.init_progressbar()
        # Always start from the first tab
        self.ui.tabWidget.setCurrentIndex(0)
        # Show it !
        self.show()

    @property
    def plottable_quant(self):
        pq = ['Frame number', 'Time', 'Position (x, right)',
              'CL velocity (x, left)', 'CL velocity (x, right)',
              'Volume (est.)',
              'Position (x, left)', 'Position (x, center)',
              'CA (right)', 'CA (left)', 'CA (mean)', 'Base radius',
              'Height', 'Area', 'Volume']
        if self.dsa.fit_method == 'wetting ridge':
            pq += ['Ridge height (left)', 'Ridge height (right)',
                   'CA (TP, left)', 'CA (TP, right)',
                   'CA (TP, mean)']
        return pq

    def init_progressbar(self):
        # Add Progress bar to status bar
        self.ui.progressbar = QtWidgets.QProgressBar()
        self.ui.progressbar.setMaximumSize(QtCore.QSize(250, 16777215))
        self.ui.progressbar.setTextVisible(True)
        self.ui.progressbar.setFormat("%p%")
        self.ui.progressbar.setValue(0)
        self.ui.progressbar.setVisible(False)
        self.ui.cancelbutton = QtWidgets.QPushButton()
        self.ui.cancelbutton.setText('Cancel')
        self.ui.cancelbutton.setVisible(False)
        self.ui.cancelbutton.clicked.connect(self.cancel_computation)
        self.ui.statusbar.addPermanentWidget(self.ui.cancelbutton)
        self.ui.statusbar.addPermanentWidget(self.ui.progressbar)

    def cancel_computation(self):
        self.dsa.stop = True

    def enable_options(self):
        """ To run after importing at least an image"""
        for tab in self.tabs:
            tab.enable_options()

    def change_tab(self, tab_nmb):
        # Do nothing if no imported image yet
        if not self.dsa.is_initialized():
            return None
        # Leave the current tab
        success = self.tabs[self.last_tab].leave_tab()
        if not success:
            self.ui.tabWidget.setCurrentIndex(self.last_tab)
            self.log.log('Something wrong happened during tab switching',
                         level=3)
            return None
        # enter the new tab
        self.tabs[tab_nmb].enter_tab()
        # Update last tab number
        self.last_tab = tab_nmb

    def set_current_ind(self, ind):
        if self.dsa.is_valid_ind(ind):
            self.current_ind = ind

    # Menu
    def export_as_script(self):
        params_im = self.tab1_get_params()
        params_edges = self.tab2_get_params()
        params_fit = self.tab3_get_params()
        raise Exception('Not implemented yet')


def run():
    # Parse args
    description = "Drop shape analyser"
    argparser = argparse.ArgumentParser(prog="PyDSA",
                                        description=description)
    args = argparser.parse_args()
    # Create apps
    app = QApplication([""])
    app.setStyle('fusion')
    w = AppWindow(app)
    w.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    run()
